namespace BookWorm
{
	public class BookWormWindow : Gtk.Window
	{
		LibraryView library_view;
		EBookArea ebook_area;

		public BookWormWindow (string path = "", Granite.Application app) {
			Gtk.Widget win_toolbar = null;
			new WindowToolbar (this, out win_toolbar);

			library_view = new LibraryView (this);
			ebook_area = new EBookArea ();

			set_title ("BookWorm");
			set_size_request (200, 200);
			default_width = 600;
			default_height = 750;

			var content_grid = new Gtk.Grid ();
			content_grid.expand = true;
			add (content_grid);

			library_view.book_selected.connect((id) => {
				open (BookWorm.get_data_dir () + "ebooks/" + id.to_string ());
			});
			library_view.open_file.connect (() => {
				open_filedialog ();
			});
			
			content_grid.attach (win_toolbar, 0, 0, 1, 1);
			content_grid.attach (library_view, 0, 1, 1, 1);
			content_grid.attach (ebook_area, 0, 2, 1, 1);

			content_grid.show ();

			ebook_area.back_pressed.connect (() => {
				library_view.show_all ();
				library_view.update_toolbar ();
				library_view.start_caching ();
				ebook_area.hide ();
			});

			if (path != "") {
				open (path);
				library_view.library.request_book (path);
			} else {
				library_view.show_all ();
				library_view.start_caching ();
				library_view.update_toolbar ();
			}
		}

		private void open_filedialog () {
			string filename = "";
			var filechooser = new Gtk.FileChooserDialog ("Open a book", this,
			Gtk.FileChooserAction.OPEN, "_Cancel",
			Gtk.ResponseType.CANCEL, "_Open", Gtk.ResponseType.OK);

			if (filechooser.run () == Gtk.ResponseType.OK)
			{
				filename = filechooser.get_filename ();
			}
			filechooser.destroy ();

			if (filename != "") {
				open (filename);
			}
		}

		private void open (string filename) {
			library_view.hide ();
			ebook_area.update_toolbar ();
			ebook_area.open_book (filename);
			ebook_area.show_all();
		}

	}
}
