namespace BookWorm {

	public class BookmarkWidget : Gtk.Grid {

		public Bookmark bookmark { get; private set; }

		public BookmarkWidget (Bookmark b) {
			bookmark = b;

			Gtk.IconTheme icon_theme = Gtk.IconTheme.get_default ();
		    try {
			    Gtk.IconInfo? icon_info = icon_theme.lookup_icon ("bookmark-new-symbolic", 16, 0);
			    assert (icon_info != null);

			    Gdk.Pixbuf pixbuf = icon_info.load_symbolic ({1, 0, 0, 1});
			    var image_widget = new Gtk.Image.from_pixbuf (pixbuf);
			    attach (image_widget, 0, 0, 1, 1);
		    } catch (Error e) {
			    warning (e.message);
		    }

			var label = new Gtk.Label (b.name);
			label.margin = 3;
			attach (label, 1, 0, 1, 1);

			this.margin = 2;
		}
	}

	public class BookmarkListWidget : Gtk.Grid {

		Bookmark[] bookmarks;

		public BookmarkListWidget (Bookmark[] bookmarks) {
			this.bookmarks = bookmarks;
			expand = true;
			valign = Gtk.Align.CENTER;
			halign = Gtk.Align.CENTER;

			int y = 0;
			set_border_width (2);
			foreach (Bookmark b in bookmarks) {
				var separator = new Gtk.Separator (Gtk.Orientation.HORIZONTAL);

				if (y != 0) {
					attach (separator, 0, y, 1, 1);
					y++;
				}

				attach (new BookmarkWidget (b), 0, y, 1, 1);
				y++;
			}
			show_all ();
		}
	}
}
