namespace BookWorm {

	public class EBookArea : GtkClutter.Embed {

		public signal void back_pressed ();

		EBookWidget ebook_widget;

		GtkClutter.Actor left_arrow;
		GtkClutter.Actor right_arrow;

		int arrow_width;
		int arrow_height;

		static const int arrow_distance = 8;

		public EBookArea () {
			expand = true;
			(get_stage () as Clutter.Actor).background_color = Clutter.Color.from_string ("white");

			left_arrow = new GtkClutter.Actor ();
			var left_arrow_box = create_arrow ("data/arrow_left.svg");
			left_arrow_box.button_press_event.connect (() => {
				ebook_widget.previous_page ();
				return true;
			});
			left_arrow.contents = left_arrow_box;
			get_stage ().add_child (left_arrow);

			right_arrow = new GtkClutter.Actor ();
			var right_arrow_box = create_arrow ("data/arrow_right.svg");
			right_arrow_box.button_press_event.connect (() => {
				ebook_widget.next_page ();
				return true;
			});
			right_arrow.contents = right_arrow_box;
			get_stage ().add_child (right_arrow);
		}

		public void update_toolbar () {
			var toolbar = WindowToolbar.instance;
			toolbar.reset ();

			Gtk.Image bookmarks_img = new Gtk.Image.from_icon_name ("format-justify-fill-symbolic",
						Gtk.IconSize.MENU);
			Gtk.ToolButton bookmarks_button = new Gtk.ToolButton (bookmarks_img, null);
			bookmarks_button.clicked.connect (() => {
                var popover = new Granite.Widgets.PopOver ();

				var menu = new HamburgerMenu (null);
				menu.show_all ();
				(popover.get_child () as Gtk.Container).add (menu);
                popover.move_to_widget (bookmarks_button, true);
			});
			toolbar.pack_start (bookmarks_button);

			Gtk.Image home_img = new Gtk.Image.from_icon_name ("user-home-symbolic",
						Gtk.IconSize.MENU);
			Gtk.ToolButton home_button = new Gtk.ToolButton (home_img, null);
			home_button.clicked.connect (() => {
				back_pressed ();
			});
			toolbar.pack_start (home_button);

			Gtk.Image font_smaller_img = new Gtk.Image.from_icon_name ("format-text-smaller-symbolic",
						Gtk.IconSize.MENU);
			Gtk.ToolButton font_smaller_button = new Gtk.ToolButton (font_smaller_img, null);
			toolbar.pack_end (font_smaller_button);

			Gtk.Image font_bigger_img = new Gtk.Image.from_icon_name ("format-text-larger-symbolic",
						Gtk.IconSize.MENU);
			Gtk.ToolButton font_bigger_button = new Gtk.ToolButton (font_bigger_img, null);
			toolbar.pack_end (font_bigger_button);

			var searchbar = new Granite.Widgets.SearchBar (_("Search this book..."));

			searchbar.text_changed_pause.connect (() => {
				ebook_widget.search (searchbar.text);
			});

			searchbar.search_icon_release.connect (() => {
				ebook_widget.search (searchbar.text);
			});

			searchbar.activate.connect (() => {
				ebook_widget.search (searchbar.text);
			});

			Gtk.ToolItem searchbar_wrapper = new Gtk.ToolItem ();
			searchbar_wrapper.add (searchbar);
			toolbar.pack_end (searchbar_wrapper);




		}

		public void open_book (string filename) {
			var reader = new GtkClutter.Actor ();
			var epub_widget = new EPubWidget();
			ebook_widget = epub_widget;
			reader.contents = epub_widget;
			epub_widget.expand = true;

			size_allocate.connect ((size) => {
				left_arrow.x = arrow_distance;
				left_arrow.y = get_allocated_height () / 2 - arrow_height / 2;

				right_arrow.x = get_allocated_width () - arrow_distance - arrow_width;
				right_arrow.y = get_allocated_height () / 2 - arrow_height / 2;

				reader.x = 40;
				reader.y = 20;
				reader.contents.set_size_request (get_allocated_width () - 80, get_allocated_height () - 40);
				queue_draw ();
				base.size_allocate (size);
			});
			get_stage ().add_child (reader);
			ebook_widget.open_book (filename);
			ebook_widget.show_all ();
		}

		public Gtk.EventBox create_arrow (string img_path) {

			var result = new Gtk.EventBox ();

			var color_white = Gdk.RGBA ();
			color_white.parse ("white");

			var arrow_img = new Gtk.Image.from_file (img_path);
			arrow_img.override_background_color (Gtk.StateFlags.NORMAL, color_white);
			result.add (arrow_img);

			arrow_width = arrow_img.pixbuf.width;
			arrow_height = arrow_img.pixbuf.height;

			result.show_all ();

			return result;

		}
	}

}
