namespace BookWorm
{
	class EPubWidget : EBookWidget {

		WebKit.WebView text_view;
		Gtk.ScrolledWindow scrolled_window;
		Gee.HashMap<string, string> contents;
		List<string> toc;
		int pageCount = 0;
		int pageno = 0;

		int index_num = 0;
		List<int> page_count_section;
		int total_page_count;
		int total_page_number;

		bool two_pages = false;
		bool calculating_total_page_count = false;
		int calc_progress = 0;

		int one_page_to_two_threshold = 1200;

		double default_animation_speed = 0.5;

		public EPubWidget() {
			text_view = new WebKit.WebView ();
			text_view.settings.enable_default_context_menu = false;
			text_view.settings.default_font_family = "droid-serif";
			text_view.expand = true;
			scrolled_window = new Gtk.ScrolledWindow (null, null);
			scrolled_window.add (this.text_view);
			scrolled_window.expand = true;
			text_view.notify["load-status"].connect (webview_status_changed);

			attach(scrolled_window,0,0,1,1);
		}

		public override void size_allocate (Gtk.Allocation allocation) {
			two_pages = (text_view.get_allocated_width ()  > one_page_to_two_threshold);
			update_page (0);
			base.size_allocate (allocation);
		}

		public override void next_page () {
			pageno++;
			total_page_number++;
			if((pageCount - 1) < pageno) {
				pageno = 0;
				index_num++;
				text_view.load_string (contents[toc.nth_data (index_num)], "text/html", "UTF-8", "file://");
			}

			update_page (default_animation_speed);
		}

		public override void previous_page () {
			pageno--;
			total_page_number--;
			if (pageno < 0) {
				pageno = 0;
				index_num--;
				text_view.load_string (contents[toc.nth_data (index_num)], "text/html", "UTF-8", "file://");
			}

			update_page (default_animation_speed);
		}
		
		public override void search (string text) {
			text_view.search_text (text, false, true, true);
		}

		private void update_page (double animation_speed) {

			string height = (text_view.get_allocated_height () - 20).to_string ();
			string width = (text_view.get_allocated_width () - 15).to_string ();

			int offset = (text_view.get_allocated_width () - 15) * pageno;

			if(two_pages) {
			text_view.execute_script (@"

				content.style['-webkit-transition-duration'] = '$animation_speed s';

				pageCount = $pageCount * 2;
				content.style.width = $width * pageCount / 2;
				content.style.height = $height;
				content.style.right = $offset;
				content.style.WebkitColumnCount =  pageCount;
				");
			} else {
				text_view.execute_script (@"

				content.style['-webkit-transition-duration'] = '$animation_speed s';

				pageCount = $pageCount;
				content.style.width = $width * pageCount;
				content.style.height = $height;
				content.style.right = $offset;
				content.style.WebkitColumnCount =  pageCount;
				");
			}
		}


		private void webview_status_changed (Object? source, ParamSpec pspec) {
			int height = (text_view.get_allocated_height () - 10);
			string width = text_view.get_allocated_width ().to_string ();
			int min;
			int natural;

			text_view.get_preferred_height (out min, out natural);

			pageCount = (int) Math.floor (min / height) + 1;
			switch (text_view.load_status) {
				case WebKit.LoadStatus.FINISHED:

					string js = @"
					var desiredHeight = $height;
					var desiredWidth = $width;

					var bodyID = document.getElementsByTagName('body')[0];

					org_html = bodyID.innerHTML;
					new_html = \"<div id = 'container'><div id='content'>\"  +
					org_html + '</div></div>';
					bodyID.innerHTML = new_html;

					var container = document.getElementsByTagName('div')[0];
					var content = document.getElementsByTagName('div')[1];


					content.style.padding = 0;

					content.style.right = 0;
					content.style.position = 'relative';

					content.style.overflow = 'visible';


					container.style.height = '100%';
					container.style.width = '100%';

					container.style.overflow = 'hidden';";

					text_view.execute_script (js);
					update_page (0);
					if (calculating_total_page_count) {

						page_count_section.append (pageCount);
						total_page_count += pageCount;
						calc_progress++;

						if (calc_progress < toc.length ()) {
						    text_view.load_string (contents[toc.nth_data (calc_progress)], "text/html", "UTF-8", "file://");
						} else {
							calculating_total_page_count = false;
							//load the current page from table of contents
							text_view.load_string (contents[toc.nth_data (index_num)], "text/html", "UTF-8", "file://");
						}
					}
					break;

			}

		}

		private void calculate_total_page_count () {
			calculating_total_page_count = true;
			text_view.load_string (contents[toc.nth_data (calc_progress)], "text/html", "UTF-8", "file://");
		}

		public override void open_book (string filename) {
			EPub.Context ebook = new EPub.Context (filename, 3);

			//get table of contents from epub
			toc = new List<string> ();

			EPub.TIterator it = new EPub.TIterator (ebook, EPub.TIteratorType.NAVMAP, 3);


			if (it == null) {
				it = new EPub.TIterator(ebook, EPub.TIteratorType.GUIDE, 3);
			}

			while (it != null && it.next () == 1) {
				toc.append (it.get_curr_link ());
			}

			//if table of contents turns out to be empty, we generate one
			bool generatetoc = (toc.length () == 0 );


			//get contents of epub
			EPub.Iterator I = new EPub.Iterator (ebook, EPub.IteratorType.SPINE, 3);
			contents = new Gee.HashMap<string, string> ();
			string next = "";

			do {
				if (generatetoc) {
					toc.append (I.get_curr_url ());
				}

				contents.set (I.get_curr_url (), next);
				next = I.get_next ();

			} while (next != null);

			calculate_total_page_count ();

		}
	}
}
