namespace BookWorm {

	public class Book {


		int _id;
		public int id {
						get {
							return _id;
						}
				}

		string _name;
		public string name {
						get {
							return _name;
						}
						set {
							_name = value;
							internal_data_changed ();
						}
		}

		string _desc;
		public string desc {
						get {
							return _desc;
						}
						set {
							_desc = value;
							internal_data_changed ();
						}
		}

		string _author;
		public string author {
						get {
							return _author;
						}
						set {
							_author = value;
							internal_data_changed ();
						}
		}

		string _tags;
		public string tags {
						get {
							return _tags;
						}
						set {
							_tags = value;
							internal_data_changed ();
						}
		}

		string _bookgroup;
		public string bookgroup {
						get {
							return _bookgroup;
						}
						set {
							_bookgroup = value;
							internal_data_changed ();
						}
		}
		
		public string display_bookgroup {
						get {
							if (bookgroup == "") {
								return _("Ungrouped");
							}
							else {
								return bookgroup;
							}
						}
		}

		string _language;
		public string language {
						get {
							return _language;
						}
						set {
							_language = value;
							internal_data_changed ();
						}
		}

		string _publication_date;
		public string publication_date {
						get {
							return _publication_date;
						}
						set {
							_publication_date = value;
							internal_data_changed ();
						}
		}

		string _isbn;
		public string isbn {
						get {
							return _isbn;
						}
						set {
							_isbn = value;
							internal_data_changed ();
						}
		}

		string _restore_data;
		public string restore_data {
						get {
							return _restore_data;
						}
						set {
							_restore_data = value;
							internal_data_changed ();
						}
		}

		string _hash;
		public string hash {
						get {
							return _hash;
						}
						set {
							_hash = value;
							internal_data_changed ();
						}
		}

		Gdk.Pixbuf _cover;
		public Gdk.Pixbuf cover {
						get {
							if(_cover == null)
								return BookWorm.get_default_cover ();
							return _cover;
						}
						set {
							_cover = value;
							internal_cover_changed ();
						}
		}

		public Book(int id, string name, string desc, string author,
						string tags, string bookgroup, string language, string
						publication_date, string isbn, string restore_data,
						string hash) {
			_id = id;
			_name = name;
			_desc = desc;
			_author = author;
			_tags = tags;
			_bookgroup = bookgroup;
			_language = language;
			_publication_date = publication_date;
			_isbn = isbn;
			_restore_data = restore_data;
			_hash = hash;
		}

		public Book.from_epub (int id, string path) {
			EPub.Context ebook = new EPub.Context (path, 3);
			_id = id;
			_name = try_to_get_info(ebook.get_metadata (EPub.Metadata.TITLE));
			_desc = try_to_get_info(ebook.get_metadata (EPub.Metadata.DESCRIPTION));
			_author = try_to_get_info(ebook.get_metadata (EPub.Metadata.CREATOR));
			_tags = "";
			_bookgroup = "";
			_language = try_to_get_info(ebook.get_metadata (EPub.Metadata.LANG));
			_publication_date = try_to_get_info(ebook.get_metadata (EPub.Metadata.DATE));
			_isbn = "";
			_restore_data = "";
			_hash = File.new_for_path (path).hash ().to_string ();
		}

		private static string try_to_get_info(string[] metadata) {
			if(metadata.length == 0) {
				return "";
			}

			string result = "";
			foreach(string s in metadata) {
				result += s + ", ";
			}

			return result.substring(0,result.length - 2);
		}

		public string to_string () {
			string result = "";
			result += id.to_string () + "|";
			result += name + "|";
			result += desc + "|";
			result += author + "|";
			result += tags + "|";
			result += bookgroup + "|";
			result += language + "|";
			result += publication_date + "|";
			result += isbn + "|";
			result += restore_data + "|";
			result += hash;
			return result;
		}

		private void internal_data_changed () {
			Idle.add(() => {
				data_changed ();
				return false;
			});
		}

		private void internal_cover_changed () {
			Idle.add(() => {
				cover_changed ();
				return false;
			});
		}

		public signal void data_changed ();

		public signal void cover_changed ();

		public void request_db_update () {
			Idle.add(() => {
				update_db_entry ();
				return false;
			});
		}

		public signal void update_db_entry ();

		public void reload_cover () {
			try{
				cover = new Gdk.Pixbuf.from_file_at_scale (get_icon_path (),
							LibraryView.ICON_W, LibraryView.ICON_H, true);
			}
			catch {
				debug ("Couldn't load cover for " + to_string ());
			}
		}

		private string get_icon_path () {
			return BookWorm.get_data_dir () + "icon_lib/" + id.to_string ();
		}

		public bool simple_search (string token) {
			if(token.down () in name.down ())
				return true;
			if(token.down () in desc.down ())
				return true;
			if(token.down () in author.down ())
				return true;
			if(token.down () in tags.down ())
				return true;
			if(token.down () in bookgroup.down ())
				return true;
			if(token.down () in isbn.down ())
				return true;
			return false;
		}
	}
}
