namespace BookWorm {

	public class LibraryGrid : LibraryViewMode {

		Gee.HashMap<string, Gtk.Grid> category_labels = new Gee.HashMap<string, Gtk.Grid> ();
		Gee.HashMap<string, Gtk.ListStore> category_store = new Gee.HashMap<string, Gtk.ListStore> ();
		Gee.HashMap<string, Gtk.IconView> category_views = new Gee.HashMap<string, Gtk.IconView> ();

		Gee.ArrayList<string> category_names = new Gee.ArrayList<string> ();

		int ITEM_WIDTH = 80;
		int ITEM_PADDING = 10;

		LibraryView parent_view;

		public LibraryGrid (LibraryView cparent_view) {
			parent_view = cparent_view;

			size_allocate.connect (() => {
				var width = get_allocated_width ();
				recalculate_columns (width);
			});
		}

		private void create_group (string name) {
			foreach (string c in category_names) {
				if (c == name)
					return;
			}

			category_names.add(name);

			int i = category_names.size - 1;

			var store = new Gtk.ListStore (4, typeof (Gdk.Pixbuf), typeof (string),
				typeof (int), typeof (bool));
			store.set_sort_column_id (1, Gtk.SortType.ASCENDING);

			var category_label = new Gtk.Label (category_names.get (i));

			category_label.margin_left = 12;
			category_label.margin_right = 8;
			var filtered = new Gtk.TreeModelFilter (store, null);
			filtered.set_visible_column(3);
			filtered.refilter ();

			var category_plugs = new Gtk.IconView.with_model (filtered);
			category_plugs.set_item_width (ITEM_WIDTH);
			category_plugs.set_text_column (1);
			category_plugs.set_pixbuf_column (0);
			category_plugs.set_hexpand (true);

			category_plugs.button_press_event.connect ((e) => {
				Gtk.TreePath path;
				Gtk.CellRenderer cell;
				category_plugs.get_item_at_pos ((int) e.x, (int) e.y, out path, out cell);
				if(path == null)
					return false;
				Gtk.TreeIter selected_book;
				store.get_iter (out selected_book, path);

				GLib.Value id;
				store.get_value (selected_book, 2, out id);

				if(e.button == 1)
					open_book (id.get_int ());
				else
					open_property_dialog (id.get_int ());
				return false;
			});

			(category_plugs.get_cells ().nth_data (0) as Gtk.CellRendererText).wrap_mode = Pango.WrapMode.WORD;
			(category_plugs.get_cells ().nth_data (0) as Gtk.CellRendererText).ellipsize_set = true;

			var bg_css = new Gtk.CssProvider ();
			try {
				bg_css.load_from_data ("*{background-color:@background_color;}", -1);
			} catch (Error e) { warning (e.message); }
			category_plugs.get_style_context ().add_provider (bg_css, 20000);
			(category_label as Gtk.Misc).xalign = 0.02f;

			var grid = new Gtk.Grid ();
			category_label.use_markup = true;

			// Always add a Seperator
			var h_separator = new Gtk.Separator (Gtk.Orientation.HORIZONTAL);
			h_separator.set_hexpand (true);
			h_separator.margin_right = 12;
			grid.attach (category_label, 0, 0, 1, 1);
			grid.attach (h_separator, 1, 0, 1, 1); // expand, fill, padding´

			grid.attach (category_plugs, 0, 1, 2, 1);

			category_labels[category_names.get (i)] = grid;
			category_store[category_names.get (i)] = store;
			category_views[category_names.get (i)] = category_plugs;

			attach (grid, 0, i, 1, 1);
		}

		public override void add_book (Book book) {
				Gtk.TreeIter root;
				string group = book.bookgroup;
				
				if (group == "")
					group = _("Ungrouped");
				
				create_group (group);
				category_store[group].append (out root);

				book.cover_changed.connect (() => {
					category_store[group].set (root, 0, book.cover, 1,
								book.name, 2, book.id, 3, true);
					queue_draw ();
				});

				book.data_changed.connect (() => {
					category_store[group].set (root, 0, book.cover, 1,
								book.name, 2, book.id, 3, true);
					queue_draw ();
				});

				category_store[group].set (root, 0, book.cover, 1,
									book.name, 2, book.id, 3, true);
				category_labels[group].show_all ();
				category_views[group].show_all ();

		}

		private void recalculate_columns (int width) {
			foreach (var view in category_views.values) {
				view.set_columns (width / (ITEM_WIDTH + ITEM_PADDING * 2));
			}
		}

		public override bool filter_books (Library library, string filter) {

			var any_found = false;
			foreach (string category in category_names) {

				var store = category_store[category];
				var container = category_labels[category];

				int shown = 0;

				store.foreach ((model, path, iter) => {
					int search_id;

					store.get (iter, 2, out search_id);

					Book b = library.get_book (search_id);

					if (b.simple_search (filter)) {
						store.set_value (iter, 3, true);
						shown ++;
					} else {
						store.set_value (iter, 3, false);
					}

					return false;
				});

				if (shown == 0) {
					container.hide ();
				} else {
					any_found = true;
					container.show ();
				}
			}
			
			visible = any_found;
			
			return any_found;
		}
	}
}
