namespace BookWorm {

	public class LibraryListView : LibraryViewMode {

		HashTable<string, ListSection> sections = new HashTable<string, ListSection> (str_hash, str_equal);

		int last_y = 0;


		public LibraryListView () {
		}

		public override void add_book (Book book) {
			string dummy_name;
			ListSection current_section;
			sections.lookup_extended (book.display_bookgroup, out dummy_name, out current_section);

			if (current_section == null) {
				current_section = new ListSection (book.display_bookgroup);
				attach (current_section, 0, last_y, 1, 1);

				last_y++;
				sections.set (book.display_bookgroup, current_section);
			}

			current_section.add_book (this, book);
			current_section.show_all ();
		}

		public override bool filter_books (Library library, string filter) {
			bool found_anything = false;
			foreach (ListSection sec in sections.get_values ()) {
				if (sec.filter_books (filter))
					found_anything = true;
			}

			visible = found_anything;

			return found_anything;
		}

	}

	public class ListSection : Gtk.Grid {

		int last_y = 1;

		ListEntry[] entries = {};

		public ListSection (string name) {
			var name_lbl = new Gtk.Label (name);
			name_lbl.margin_left = 12;
			name_lbl.margin_right = 8;
			attach (name_lbl, 0, 0, 1, 1);

			var h_separator = new Gtk.Separator (Gtk.Orientation.HORIZONTAL);
			h_separator.set_hexpand (true);
			h_separator.margin_right = 12;
			attach (h_separator, 1, 0, 1, 1);

			margin_bottom = 20;
		}

		public bool filter_books (string filter) {
			bool found_anything = false;
			foreach (ListEntry e in entries) {
				if (e.filter_books (filter))
					found_anything = true;
			}

			//visible = found_anything;

			return found_anything;
		}

		public void add_book (LibraryViewMode mode, Book b) {
			var entry = new ListEntry (b);

			var event_box = new Gtk.EventBox ();
			event_box.button_press_event.connect ((e) => {
				if(e.button == 1)
					mode.open_book (b.id);
				else
					mode.open_property_dialog (b.id);
				return true;
			});

			event_box.add (entry);

			attach (event_box, 0, last_y, 2, 1);
			last_y++;
			entries += entry;
		}
	}

	public class ListEntry : Gtk.Grid {

		Book book;

		public ListEntry (Book b) {
			book = b;
			margin_top = 15;
			margin_left = 19;

			Gtk.Image cover = new Gtk.Image.from_pixbuf (b.cover);
			var cover_box = new Gtk.EventBox ();
			cover_box.width_request = LibraryView.ICON_W;
			cover_box.height_request = LibraryView.ICON_H;
			cover_box.add (cover);
			book.cover_changed.connect (() => {
				Gtk.Image new_cover = new Gtk.Image.from_pixbuf (b.cover);
				cover_box.remove (cover_box.get_child ());
				cover_box.add (new_cover);
				show_all ();
			});

			attach (cover_box, 0, 0, 1, 4);

			var name_lbl = new Gtk.Label ("");
			name_lbl.set_markup (@"<b><big>$(book.name)</big></b>");
			name_lbl.margin_left = 5;
			attach (name_lbl, 1, 0, 1, 1);

			var author_lbl = new Gtk.Label (create_author_text ());
			//author_lbl.m_wrap_width = 200;
			attach (author_lbl, 2, 0, 1, 1);


			var desc_lbl = new Gtk.Label (book.desc);
			desc_lbl.margin_left = 5;
			attach (desc_lbl, 1, 1, 1, 1);

		}

		public bool filter_books (string filter) {
			visible = book.simple_search (filter);

			return visible;
		}

		private string create_author_text () {
			return _(" by $AUTHOR")
				.replace ("$AUTHOR", book.author);
		}

	}



}
