namespace BookWorm {

	public class LibraryView : Gtk.Grid {

		LibraryGrid grid_view;
		LibraryListView list_view;

		Gtk.ScrolledWindow grid_view_scrollable;
		Gtk.ScrolledWindow list_view_scrollable;

		NotFoundScreen not_found_grid = new NotFoundScreen ();
		NotFoundScreen not_found_list = new NotFoundScreen ();

		public Library library { get; private set; }

		int _view_mode = 0;
		public int view_mode {
			get {
				return _view_mode;
			}
			set {
				switch (value) {
				case 0:
					grid_view_scrollable.show ();
					list_view_scrollable.hide ();
					break;
				case 1:
					grid_view_scrollable.hide ();
					list_view_scrollable.show ();
					break;
				default:
					error ("Invalid view-mode parameter:" + value.to_string ());
				}
				_view_mode = value;
			}
		}

		public signal void book_selected (int id);
		public signal void open_file ();

		public static int ICON_W = 60;
		public static int ICON_H = 80;

		bool started_loading = false;

		Gtk.Window parent_window;

		public LibraryView (Gtk.Window cparent) {
			parent_window = cparent;
			library = new Library ();
			library.cache_books_async ();

			grid_view = new LibraryGrid (this);
			grid_view.margin_top = 5;
			grid_view.expand = true;
			grid_view.open_book.connect ((id) => {
				book_selected (id);
			});
			grid_view.open_property_dialog.connect ((id) => {
				open_property_dialog (id);
			});

			var grid_view_container = new Gtk.Grid ();
			grid_view_container.attach (grid_view, 0, 0, 1, 1);
			grid_view_container.attach (not_found_grid, 0, 1, 1, 1);

			grid_view_scrollable = new Gtk.ScrolledWindow (null, null);
			grid_view_scrollable.add_with_viewport (grid_view_container);
			grid_view_scrollable.hscrollbar_policy = Gtk.PolicyType.NEVER;
			grid_view.set_vexpand (true);
			attach (grid_view_scrollable, 0, 1, 1, 1);

			list_view = new LibraryListView ();
			list_view.margin_top = 5;
			list_view.expand = true;
			list_view.open_book.connect ((id) => {
				book_selected (id);
			});
			list_view.open_property_dialog.connect ((id) => {
				open_property_dialog (id);
			});

			var list_view_container = new Gtk.Grid ();
			list_view_container.attach (list_view, 0, 0, 1, 1);
			list_view_container.attach (not_found_list, 0, 1, 1, 1);

			list_view_scrollable = new Gtk.ScrolledWindow (null, null);
			list_view_scrollable.add_with_viewport (list_view_container);
			list_view_scrollable.hscrollbar_policy = Gtk.PolicyType.NEVER;
			list_view.set_vexpand (true);
			attach (list_view_scrollable, 0, 2, 1, 1);
		}

		public void start_caching () {
			if (!started_loading) {
				update_view_from_db ();
				Gdk.threads_add_timeout (300, update_view_from_db);
				started_loading = true;
			}
		}

		public void update_toolbar () {
			var toolbar = WindowToolbar.instance;
			toolbar.reset ();

			var modebutton = new Granite.Widgets.ModeButton ();
			modebutton.append_icon ("view-grid-symbolic", Gtk.IconSize.MENU);
			modebutton.append_icon ("view-list-compact-symbolic", Gtk.IconSize.MENU);

			modebutton.mode_changed.connect (() => {
				view_mode = modebutton.selected;
			});
			modebutton.selected = 0;

			var cont_mode = new Gtk.ToolItem ();
			cont_mode.add (modebutton);
			cont_mode.margin = 2;
			toolbar.pack_start (cont_mode);

			Gtk.Image open_img = new Gtk.Image.from_icon_name ("document-open",
						Gtk.IconSize.MENU);
			Gtk.ToolButton open_button = new Gtk.ToolButton (open_img, null);
			open_button.clicked.connect (() => {
				open_file ();
			});
			toolbar.pack_start (open_button);

			var searchbar = new Gtk.SearchEntry ();
			searchbar.placeholder_text = _("Search Books");

			searchbar.changed.connect(() => {
				bool found_anything = grid_view.filter_books (library, searchbar.get_text ())
				|| list_view.filter_books (library, searchbar.get_text ());
				
				not_found_grid.visible = not_found_list.visible = searchbar.text != "";
				
				not_found_grid.show_separator (found_anything);
				not_found_list.show_separator (found_anything);
			});
			var cont = new Gtk.ToolItem ();
			cont.add (searchbar);
			cont.margin = 2;
			toolbar.pack_end (cont);
		}

		private bool update_view_from_db () {
			var list = library.get_delta_books ();
			foreach (Book b in list) {
				grid_view.add_book (b);
				list_view.add_book (b);
			}

			return !library.is_ready ();
		}

		public void open_property_dialog (int id) {
			Book b = library.get_book (id);
			new PropertyDialog(parent_window, library, b);
		}
	}
}
