namespace BookWorm {

	public class PropertyDialog : Granite.Widgets.LightWindow {

		Library library;
		Book book;
		Gtk.EventBox cover_box;
		bool loading_cover = false;


		public PropertyDialog(Gtk.Window parent, Library lib, Book b) {
			library = lib;
			book = b;
			set_transient_for (parent);
			set_title (b.name);

			Gtk.Grid main_grid = new Gtk.Grid ();

			Gtk.Image cover = new Gtk.Image.from_pixbuf (b.cover);
			cover_box = new Gtk.EventBox();
			cover_box.width_request = LibraryView.ICON_W;
			cover_box.height_request = LibraryView.ICON_H;
			cover_box.button_press_event.connect (() => {
				request_cover_change ();
				return true;
			});
			cover_box.add (cover);
			book.cover_changed.connect (() => {
				Gtk.Image new_cover = new Gtk.Image.from_pixbuf (b.cover);
				cover_box.remove (cover_box.get_child ());
				cover_box.add (new_cover);
				show_all ();
			});


			Gtk.Grid cover_grid = new Gtk.Grid ();
			cover_grid.attach (cover_box, 1, 1, 1, 1);
			main_grid.attach (cover_grid, 0, 0, 1, 3);

			main_grid.attach (make_field (N_("Name")), 1, 0, 1, 1);
			main_grid.attach (make_field (N_("Author")), 1, 1, 1, 1);
			main_grid.attach (make_field (N_("ISBN")), 0, 2, 2, 1);
			main_grid.attach (make_field (N_("Group")), 0, 3, 2, 1);

			Gtk.Separator separator_top_desc = new Gtk.Separator (Gtk.Orientation.HORIZONTAL);
			separator_top_desc.hexpand = true;
			main_grid.attach (separator_top_desc, 0, 4, 2, 1);
			main_grid.attach (make_field (N_("Description")), 0, 5, 2, 1);

			Gtk.Separator separator_bot_desc = new Gtk.Separator (Gtk.Orientation.HORIZONTAL);
			separator_bot_desc.hexpand = true;
			main_grid.attach (separator_bot_desc, 0, 6, 2, 1);

			main_grid.margin = 8;
			add (main_grid);

			this.delete_event.connect (() => {
				b.request_db_update ();
				return false;
			});

			show_all ();
		}

		private Gtk.Grid make_field (string name) {
			Gtk.Grid result = new Gtk.Grid ();

			Gtk.Grid spanner = new Gtk.Grid();
			spanner.hexpand = true;

			Gtk.Label label = new Gtk.Label (_(name));
			label.margin = 5;
			result.attach (label, 1, 0, 1, 1);

			Gtk.Widget value_field = null;
			switch(name) {
				case N_("Name"):
					Gtk.Entry entry = new Gtk.Entry ();
					entry.text = book.name;
					entry.changed.connect (() => {
						book.name = entry.text;
					});
					value_field = entry;
					result.attach (spanner, 0, 0, 1, 1);
					break;
				case N_("Description"):
					Gtk.TextView entry = new Gtk.TextView ();
					entry.margin = 4;
					entry.expand = true;
					entry.buffer.text = book.desc;
					entry.buffer.changed.connect (() => {
						book.desc = entry.buffer.text;
					});
					value_field = entry;
				break;
				case N_("Group"):
					Gtk.Entry entry = new Gtk.Entry ();
					entry.text = book.bookgroup;
					entry.changed.connect (() => {
						book.bookgroup = entry.text;
					});
					value_field = entry;
					result.attach (spanner, 0, 0, 1, 1);
				break;
				case N_("Author"):
					Gtk.Entry entry = new Gtk.Entry ();
					entry.text = book.author;
					entry.changed.connect (() => {
						book.author = entry.text;
					});
					value_field = entry;
					result.attach (spanner, 0, 0, 1, 1);
				break;
				case N_("ISBN"):
					Gtk.Entry entry = new Gtk.Entry ();
					entry.text = book.isbn;
					entry.changed.connect (() => {
						book.isbn = entry.text;
					});
					value_field = entry;
					result.attach (spanner, 0, 0, 1, 1);
				break;
			}
			value_field.margin = 4;
			result.attach (value_field, 2, 0, 1, 1);
			result.show_all ();
			return result;
		}

		public void request_cover_change () {
			if(loading_cover)
				return;
			loading_cover = true;


			string filename = "";
			var filechooser = new Gtk.FileChooserDialog ("Title", this,
					Gtk.FileChooserAction.OPEN, "_Cancel",
					Gtk.ResponseType.CANCEL, "_Open", Gtk.ResponseType.OK);

			if (filechooser.run () == Gtk.ResponseType.OK)
			{
				filename = filechooser.get_filename ();
			}
			filechooser.destroy ();

			if (filename == "") {
				loading_cover = false;
				return;
			}

			File file1 = File.new_for_path (filename);
			File file2 = File.new_for_path (BookWorm.get_data_dir()
									+ "icon_lib/" + book.id.to_string ());
			try {
				Gtk.Spinner spinner = new Gtk.Spinner ();
				spinner.width_request = LibraryView.ICON_W;
				spinner.height_request = LibraryView.ICON_H;
				cover_box.remove (cover_box.get_child ());
				cover_box.add (spinner);
				show_all ();

				file1.copy_async.begin (file2, FileCopyFlags.OVERWRITE, Priority.DEFAULT,
												null, null, (obj, res) => {
					book.reload_cover ();
					loading_cover = false;
				});
			} catch (Error e) {
				warning ("Errddor: %s\n", e.message);
			}
		}
	}
}
