namespace BookWorm {

	/**
	 * Singleton-Toolbar as a wrapper around a normal Toolbar or a HeaderBar
	 * depending on the GTK version. Note that this is not a GTK widget.
	 * Instead we pass the actual GTK Widget back via a out in the constructor.
	 */
	public class WindowToolbar {

		//current toolbar
		Gtk.Toolbar toolbar = null;

		public static WindowToolbar instance { get; private set; default = null; }

		private Gtk.Window win;

		/**
		 * Create the singleton (and fails therefore when you call this method
		 * more then one time) and returns the internal grid-widget that contains
		 * the current toolbar via the second parameter.
		 * You can either save the resulting instance or use the
		 * WindowToolbar.instance-property.
		 * The win-parameter is a GTK window that get's a HeaderBar in case
		 * we use GTK >= 3.10.
		 */
		public WindowToolbar (Gtk.Window win, out Gtk.Widget grid_out) {
			this.win = win;

			toolbar = new Gtk.Toolbar ();
			toolbar.set_hexpand (true);
			toolbar.show_all ();

			reset ();
			assert (instance == null);
			instance = this;
			grid_out = toolbar;
			//TODO add HeaderBar
		}

		/*
		 * Resets the Toolbar to a empty state.
		 */
		public void reset () {
			toolbar.@foreach ((widget) => {
				toolbar.remove (widget);
			});

			var spacer = new Gtk.ToolItem ();
			spacer.set_expand (true);
			spacer.show_all ();
			pack_start (spacer);
		}

		/**
		 * Tries to pack the item at the most left place (and always left
		 * of the title in case of a HeaderBar).
		 */
		public void pack_start (Gtk.ToolItem item) {
			toolbar.insert (item, 0);
			toolbar.show_all ();
		}

		/**
		 * Tries to pack the item at the most right place (and always right
		 * of the title in case of a HeaderBar).
		 */
		public void pack_end (Gtk.ToolItem item) {
			toolbar.add (item);
			toolbar.show_all ();
		}
	}
}
