[CCode (cheader_filename = "epub.h")]
namespace EPub {
  [CCode (cname = "epub_metadata", cprefix = "EPUB_")]
  public enum Metadata {
    ID,
    TITLE,
    CREATOR,
    CONTRIB,
    SUBJECT,
    PUBLISHER,
    DESCRIPTION,
    DATE,
    TYPE,
    FORMAT,
    SOURCE,
    LANG,
    RELATION,
    COVERAGE,
    RIGHTS,
    META
  }

  [CCode (cname = "eiterator_type", cprefix = "EITERATOR_")]
  public enum IteratorType {
    SPINE,
    LINEAR,
    NONLINEAR
  }

  [CCode (cname = "titerator_type", cprefix = "TITERATOR_")]
  public enum TIteratorType {
    NAVMAP,
    GUIDE,
    PAGES
  }

  [Compact, CCode (cname = "struct epub", free_function = "epub_close", lower_case_cprefix = "epub_")]
  public class Context {
    [CCode (cname = "epub_open")]
    public Context (string filename, int debug = 0);
    public void set_debug (int debug);
    public int get_ocf_file (string filename, out string data);
    public int close ();
    public void dump ();
    public string[] get_metadata (EPub.Metadata type);
    public int get_data (string name, out string data);
  }

  [Compact, CCode (cname = "struct eiterator", free_function = "epub_free_iterator", lower_case_cprefix = "epub_it_")]
  public class Iterator {
    [CCode (cname = "epub_get_iterator")]
    public Iterator (EPub.Context epub, EPub.IteratorType type, int opt);
    public unowned string get_next ();
    public unowned string get_curr ();
    public unowned string get_curr_url ();
  }

  [Compact, CCode (cname = "struct titerator", free_function = "epub_free_titerator", lower_case_cprefix = "epub_tit_")]
  public class TIterator {
    [CCode (cname = "epub_get_titerator")]
    public TIterator (EPub.Context epub, EPub.TIteratorType type, int opt);
    public int curr_valid ();
    public int get_curr_depth ();
    public unowned string get_curr_link ();
    public unowned string get_curr_label ();
    public int next ();
  }

  [CCode (cname = "epub_cleanup")]
  public static void cleanup ();
}